import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Popover, Col, Modal, Button } from 'antd'
import { UpOutlined, PlusOutlined, PlusSquareOutlined, EditOutlined, ToolOutlined, FontColorsOutlined } from '@ant-design/icons'

import asyncComponent from '@/utils/asyncComponent'
import asyncIconComponent from '@/utils/asyncIconComponent'
import { resetStyle } from '@/utils/utils-custom.js'
import getSettingForm from './options'
import Utils from '@/utils/utils.js'
import MKEmitter from '@/utils/events.js'
import './index.scss'

const { confirm } = Modal

const NormalForm = asyncIconComponent(() => import('@/components/normalform'))
const CardCellComponent = asyncComponent(() => import('@/menu/components/card/cardcellcomponent'))
const CopyComponent = asyncIconComponent(() => import('@/menu/components/share/copycomponent'))
const PasteController = asyncIconComponent(() => import('@/components/paste'))

class DoubleCardBoxComponent extends Component {
  static propTpyes = {
    cards: PropTypes.object,         // 卡片行配置信息
    card: PropTypes.object,          // 卡片配置信息
    updateElement: PropTypes.func    // 菜单配置更新
  }

  state = {
    card: null,            // 卡片信息，包括正反面
    appType: sessionStorage.getItem('appType'),
    visible: false,
    timestamp: '',
    side: ''
  }

  /**
   * @description 搜索条件初始化
   */
  UNSAFE_componentWillMount () {
    const { card } = this.props

    this.setState({
      card: fromJS(card).toJS(),
      timestamp: new Date().getTime() + ''
    })
  }

  shouldComponentUpdate (nextProps, nextState) {
    const { cards } = this.props
    
    return !is(fromJS(cards), fromJS(nextProps.cards)) || !is(fromJS(this.state), fromJS(nextState))
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
  }

  getStyle = (style) => {
    const { card, side } = this.state

    let _card = fromJS(card).toJS()
    if (side === 'sub') {
      _card.backStyle = style
    } else {
      _card.style = style
    }

    this.setState({
      card: _card
    })

    this.props.updateElement(_card)
  }

  updateCard = (elements, btn, type) => {
    const { card } = this.state

    let _card = {}

    if (type === 'sub') {
      _card = {...card, backElements: elements}
    } else {
      _card = {...card, elements: elements}
    }

    this.setState({
      card: _card
    })

    this.props.updateElement(_card, btn)
  }
  
  addElement = (type) => {
    const { card } = this.state

    let newcard = {}
    newcard.uuid = Utils.getuuid()
    newcard.focus = true
    
    newcard.eleType = 'text'
    newcard.datatype = 'dynamic'
    newcard.height = 1

    // 注册事件-添加元素
    MKEmitter.emit('cardAddElement', card.uuid, newcard, type)
  }

  addButton = (type) => {
    const { card } = this.state

    let newcard = {eleType: 'button', label: 'button', verify: null, show: 'link', sqlType: '', Ot: 'requiredSgl', OpenType: 'prompt', icon: '', class: 'primary', intertype: 'system', execSuccess: 'grid', execError: 'never', popClose: 'never'}
    newcard.uuid = Utils.getuuid()
    newcard.focus = true

    // 注册事件-添加元素
    MKEmitter.emit('cardAddElement', card.uuid, newcard, type)
  }

  changeStyle = (type) => {
    const { card } = this.state

    let _style = null
    let options = ['height', 'background', 'border', 'padding', 'margin', 'shadow', 'overflow']

    if (type === 'sub') {
      _style = fromJS(card.backStyle).toJS()
    } else {
      options.push('clear')
      _style = fromJS(card.style).toJS()
    }

    this.setState({side: type})

    MKEmitter.emit('changeStyle', options, _style, this.getStyle)
  }

  getSettingForms = () => {
    const { cards } = this.props
    const { card } = this.state

    let buttons = []
    card.elements.forEach(item => {
      if (item.eleType === 'button') {
        buttons.push({
          value: item.uuid,
          label: item.label
        })
      }
    })

    return getSettingForm(card.setting, buttons, cards.columns, 'main')
  }

  getBackSettingForms = () => {
    const { cards } = this.props
    const { card } = this.state

    let buttons = []
    card.backElements.forEach(item => {
      if (item.eleType === 'button') {
        buttons.push({
          value: item.uuid,
          label: item.label
        })
      }
    })

    return getSettingForm(card.backSetting, buttons, cards.subColumns)
  }

  updateSetting = (res, type) => {
    const { card, appType } = this.state

    if (appType === '' && res.menu) {
      let list = null
      try {
        list = JSON.parse(sessionStorage.getItem('thdMenuList')) || []
      } catch (e) {
        list = []
      }

      let id = res.menu[res.menu.length - 1]

      list.forEach(item => {
        if (item.MenuID === id) {
          res.MenuID = id
          res.MenuName = item.MenuName
          res.MenuNo = item.MenuNo
          res.tabType = item.type
        }
      })
    }

    let _card = {...card}

    if (type === 'sub') {
      _card.backSetting = res
    } else {
      _card.setting = res
    }

    this.setState({
      card: _card
    })

    this.props.updateElement(_card)
  }

  updateFields = (elements, resolve, position, type) => {
    const { card } = this.state

    let items = elements

    if (type === 'form') {
      items = []

      elements.forEach(item => {
        let cell = {
          uuid: Utils.getuuid(),
          eleType: 'text',
          height: 1,
          innerHeight: 21,
          prefix: item.label ? item.label + ': ' : '',
          datatype: item.field ? 'dynamic' : 'static',
          field: item.field || '',
          width: item.span || 12,
          value: '',
          style: {marginBottom: '15px'}
        }

        if (item.type === 'number') {
          cell.eleType = 'number'
          cell.decimal = typeof(item.decimal) === 'number' ? item.decimal : ''
        }

        items.push(cell)
      })
    }

    if (items.length === 0) {
      resolve({status: false, message: '元素不可为空！'})
      return
    }

    let _card = { ...card }
    let _card2 = { ...card }
    let pass = false

    if (position === 'sub') {
      if (_card.backElements.length === 0) {
        _card.backElements = items
        pass = true
      } else {
        _card.backElements = items
        _card2.backElements = [...card.backElements, ...items]
      }
    } else {
      if (_card.elements.length === 0) {
        _card.elements = items
        pass = true
      } else {
        _card.elements = items
        _card2.elements = [...card.elements, ...items]
      }
    }

    if (pass) {
      this.setState({
        card: _card,
        timestamp: new Date().getTime() + ''
      })

      this.props.updateElement(_card)
    } else {
      const that = this
      const _modal = confirm({
        title: '卡片中已存在元素，请选择要进行的操作。',
        className: 'mk-confirm-modal',
        content: <div className="footer">
          <Button key="cancel" onClick={() => _modal.destroy()}>取消</Button>
          <Button key="replace" className="mk-border-purple" onClick={() => {
            that.setState({
              card: _card,
              timestamp: new Date().getTime() + ''
            })
      
            that.props.updateElement(_card)
            _modal.destroy()
          }}>替换</Button>
          <Button key="confirm" className="mk-border-green" onClick={() => {
            that.setState({
              card: _card2,
              timestamp: new Date().getTime() + ''
            })
      
            that.props.updateElement(_card2)
            _modal.destroy()
          }}>添加</Button>
        </div>
      })
    }

    resolve({status: true})
  }

  paste = (element, resolve, type) => {
    const { card } = this.state

    if (['simpleform', 'forms', 'formgroup', 'cardcell'].includes(element.copyType)) {
      if (element.copyType === 'simpleform') {
        this.updateFields(element.subcards[0].fields, resolve, type, 'form')
      } else if (element.copyType === 'forms') {
        this.updateFields(element.fields, resolve, type, 'form')
      } else if (element.copyType === 'formgroup') {
        this.updateFields(element.fields, resolve, type, 'form')
      } else {
        this.updateFields(element.elements, resolve, type)
      }
      return
    }

    let _uuid = Utils.getuuid()
    
    if (element.copyType === 'action') {
      element.eleType = 'button'
    }

    element.uuid = _uuid
    element.focus = true

    resolve({status: true})

    // 注册事件-添加元素
    MKEmitter.emit('cardAddElement', card.uuid, element, type)
  }

  doubleClickCard = (type) => {
    const { card } = this.state

    if (type !== 'sub' && card.setting.click === 'menu' && card.setting.menu) {
      MKEmitter.emit('changeEditMenu', {MenuID: card.setting.menu})
    } else if (type === 'sub' && card.backSetting.click === 'menu' && card.backSetting.menu) {
      MKEmitter.emit('changeEditMenu', {MenuID: card.backSetting.menu})
    }
  }

  render() {
    const { cards } = this.props
    const { card, timestamp } = this.state

    let _style = {...card.style}
    let _backStyle = {...card.backStyle}
    let _wrapStyle = {}

    _style = resetStyle(_style)
    _backStyle = resetStyle(_backStyle)

    if (card.setting.position === 'inner') {
      Object.keys(_style).forEach(key => {
        if (!/^(margin|border|box)/.test(key)) return
        _wrapStyle[key] = _style[key]
        delete _style[key]
      })
    }

    let checkAll = ''
    let mainBox = ''
    if (cards.wrap.selStyle === 'check') {
      checkAll = 'mk-checkable'
    } else if (cards.wrap.selStyle === 'check square') {
      checkAll = 'mk-checkable square'
    } else if (card.setting.display !== 'default' && card.setting.controlIcon === 'left') {
      mainBox = 'flex-card '
    }

    let _style_ = null

    if (card.style.clear === 'left') {
      _style_ = {clear: 'left'}
    } else if (card.style.clear === 'right') {
      _style_ = {float: 'right'}
    }

    return (
      <Col span={card.setting.width || 24} style={_style_}>
        <div className="card-item-wrap" style={_wrapStyle}>
          <div className={`card-item ${card.setting.btnControl || ''} ${checkAll} mk-${card.setting.display} ${mainBox}`} style={_style} onDoubleClick={(e) => {e.stopPropagation(); this.doubleClickCard()}} id={card.uuid}>
            <span className="circle-select"></span>
            {card.setting.controlIcon === 'left' ? <PlusSquareOutlined /> : <UpOutlined />}
            <div className="card-control" onDoubleClick={(e) => e.stopPropagation()}>
              <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
                <div className="mk-popover-control">
                  <PlusOutlined className="plus" title="添加元素" onClick={() => this.addElement('main')} />
                  <PlusSquareOutlined className="plus" title="添加按钮" onClick={() => this.addButton('main')} />
                  <NormalForm title={'循环卡片设置'} width={950} update={(res) => this.updateSetting(res)} getForms={this.getSettingForms}>
                    <EditOutlined className="edit" title="编辑"/>
                  </NormalForm>
                  <CopyComponent type="cardcell" card={{...card, backElements: []}}/>
                  <PasteController options={['action', 'customCardElement', 'simpleform', 'forms', 'formgroup', 'cardcell']} updateConfig={(element, resolve) => this.paste(element, resolve, 'main')} />
                  <FontColorsOutlined className="style" title="调整样式" onClick={() => this.changeStyle()} />
                </div>
              } trigger="hover">
                <ToolOutlined />
              </Popover>
            </div>
            <CardCellComponent cards={cards} cardCell={card} side="main" timestamp={timestamp} elements={card.elements} updateElement={(elements, btn) => this.updateCard(elements, btn)}/>
          </div>
          <Col span={card.backSetting.width || 24}>
            <div className={'card-item ' + (card.backSetting.btnControl || '')} style={_backStyle} onDoubleClick={(e) => {e.stopPropagation(); this.doubleClickCard('sub')}} id={card.uuid}>
              <div className="card-control" onDoubleClick={(e) => e.stopPropagation()}>
                <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
                  <div className="mk-popover-control">
                    <PlusOutlined className="plus" title="添加元素" onClick={() => this.addElement('sub')} />
                    <PlusSquareOutlined className="plus" title="添加按钮" onClick={() => this.addButton('sub')} />
                    <NormalForm title={'循环子卡片设置'} width={950} update={(res) => this.updateSetting(res, 'sub')} getForms={this.getBackSettingForms}>
                      <EditOutlined className="edit" title="编辑"/>
                    </NormalForm>
                    <CopyComponent type="cardcell" card={{...card, elements: card.backElements, backElements: []}}/>
                    <PasteController options={['action', 'customCardElement', 'simpleform', 'forms', 'formgroup', 'cardcell']} updateConfig={(element, resolve) => this.paste(element, resolve, 'sub')} />
                    <FontColorsOutlined className="style" title="调整样式" onClick={() => this.changeStyle('sub')} />
                  </div>
                } trigger="hover">
                  <ToolOutlined />
                </Popover>
              </div>
              <CardCellComponent cards={cards} cardCell={card} side="sub" timestamp={timestamp} elements={card.backElements} updateElement={(elements, btn) => this.updateCard(elements, btn, 'sub')}/>
            </div>
          </Col>
        </div>
      </Col>
    )
  }
}

export default DoubleCardBoxComponent